import { WebTrackerMiniOptions, StsPlugin } from '@aliyun-sls/web-types';
import WebTracker from '@aliyun-sls/web-track-base';

/**
 * 小程序平台 SDK 接口
 */
interface PlatformSDK {
    [key: string]: any;
    request: Function;
    httpRequest?: Function;
    getSystemInfoSync: Function;
    onError?: Function;
    onUnhandledRejection?: Function;
    onPageNotFound?: Function;
    onMemoryWarning?: Function;
    getLaunchOptionsSync?: Function;
}
interface PlatformSDKInterop {
    sdk: PlatformSDK;
    appName: string;
    setStorageSync: (key: string, value: string) => void;
    getStorageSync: (key: string) => string | null | undefined;
    getCurrentPagesInterop: () => PageObject[];
    getNavigateBackUrl: (delta: number) => string;
    getCurrentUrl: () => string;
}
interface PageObject {
    route: string;
    options: Record<string, any>;
}
/**
 * 获取跨平台的 SDK
 */
declare const getPlatformSDKInterop: (opt: WebTrackerMiniOptions) => PlatformSDKInterop;
declare function getPlatformRequestMethod(sdk: PlatformSDK, platformRequestName?: string): {
    request: any;
    requestName: string;
};

declare class WebTrackerMini extends WebTracker {
    constructor(opt: WebTrackerMiniOptions);
    useStsPlugin(stsPlugin: StsPlugin): void;
}

export { PlatformSDK, PlatformSDKInterop, WebTrackerMini as default, getPlatformRequestMethod, getPlatformSDKInterop };
