function getPageUrl(url, options = {}) {
  const queryItems = [];
  Object.keys(options).forEach((k) => {
    queryItems.push(`${k}=${options[k]}`);
  });
  if (queryItems.length === 0) {
    return `/${url}`;
  }
  return `/${url}${url.indexOf("?") !== -1 ? "&" : "?"}${queryItems.join("&")}`;
}

var __defProp = Object.defineProperty;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
function createSDKInterop(sdk) {
  const setStorageSync = (key, value) => {
    if (typeof sdk.setStorageSync === "function") {
      sdk.setStorageSync(key, value);
    }
  };
  const getStorageSync = (key) => {
    if (typeof sdk.getStorageSync === "function") {
      return sdk.getStorageSync(key);
    }
    return null;
  };
  const getCurrentPagesInterop = () => {
    if (typeof getCurrentPages !== "function") {
      return [];
    }
    return getCurrentPages();
  };
  const getNavigateBackUrl = (delta = 1) => {
    const pages = getCurrentPages();
    if (!pages.length) {
      return "";
    }
    const toPage = pages[pages.length - delta];
    return getPageUrl(toPage.route, toPage.options);
  };
  const getCurrentUrl = () => {
    const pages = getCurrentPagesInterop();
    if (!pages.length) {
      return "";
    }
    const currentPage = pages.pop();
    return getPageUrl(currentPage.route, currentPage.options);
  };
  return {
    getStorageSync,
    setStorageSync,
    getCurrentPagesInterop,
    getNavigateBackUrl,
    getCurrentUrl
  };
}
const getPlatformSDKInterop = (opt) => {
  let currentSdk = {
    // tslint:disable-next-line: no-empty
    request: () => {
    },
    // tslint:disable-next-line: no-empty
    httpRequest: () => {
    },
    // tslint:disable-next-line: no-empty
    getSystemInfoSync: () => {
    }
  };
  let currentAppName = "unknown";
  if (typeof wx === "object") {
    currentSdk = wx;
    currentAppName = "wechat";
  } else if (typeof dd === "object") {
    currentSdk = dd;
    currentAppName = "dingtalk";
  } else if (typeof my === "object") {
    currentSdk = my;
    currentAppName = "alipay";
  } else if (typeof tt === "object") {
    currentSdk = tt;
    currentAppName = "bytedance";
  } else if (typeof qq === "object") {
    currentSdk = qq;
    currentAppName = "qq";
  } else if (typeof swan === "object") {
    currentSdk = swan;
    currentAppName = "swan";
  } else if (opt.platformSDK) {
    currentSdk = opt.platformSDK;
  } else {
    throw new Error(
      "Current platform is not default supported by SLS API, Pleace config platformSDK or contack Aliyun SLS team."
    );
  }
  const interop = createSDKInterop(currentSdk);
  return __spreadValues({
    sdk: currentSdk,
    appName: currentAppName
  }, interop);
};
function getPlatformRequestMethod(sdk, platformRequestName) {
  let request;
  let requestName = "unknown";
  if (platformRequestName && sdk[platformRequestName]) {
    request = sdk[platformRequestName];
    requestName = platformRequestName;
  } else if (sdk.request) {
    request = sdk.request;
    requestName = "request";
  } else if (sdk.httpRequest) {
    request = sdk.httpRequest;
    requestName = "httpRequest";
  } else {
    throw new Error(
      "Current platform is not default supported by SLS API, Pleace config platformRequestName or contack Aliyun SLS team."
    );
  }
  return { request, requestName };
}

class WebTracker {
  constructor(opt) {
    this.timer = null;
    this.time = 10;
    this.count = 10;
    this.arr = [];
    var _a, _b;
    this.time = (_a = opt.time) != null ? _a : 10;
    this.count = (_b = opt.count) != null ? _b : 10;
    if (opt.host.startsWith("http://") || opt.host.startsWith("https://")) {
      this.url = opt.host + "/logstores/" + opt.logstore + "/track";
    } else {
      this.url = "https://" + opt.project + "." + opt.host + "/logstores/" + opt.logstore + "/track";
    }
    this.opt = opt;
    if (opt.installUnloadHook && typeof opt.installUnloadHook === "function") {
      opt.installUnloadHook(() => {
        this.sendImmediateInner();
      });
    }
  }
  assemblePayload(arr) {
    const payload = {
      __logs__: arr
    };
    if (this.opt.tags) {
      payload.__tags__ = this.opt.tags;
    }
    if (this.opt.topic) {
      payload.__topic__ = this.opt.topic;
    }
    if (this.opt.source) {
      payload.__source__ = this.opt.source;
    }
    return JSON.stringify(payload);
  }
  platformSend() {
    if (this.opt.sendPayload && typeof this.opt.sendPayload === "function") {
      const payloadStr = this.assemblePayload(this.arr);
      this.opt.sendPayload(this.url, payloadStr);
    }
  }
  transString(obj) {
    let newObj = {};
    for (let i in obj) {
      if (typeof obj[i] == "object") {
        newObj[i] = JSON.stringify(obj[i]);
      } else {
        newObj[i] = String(obj[i]);
      }
    }
    return newObj;
  }
  sendImmediateInner() {
    if (this.arr && this.arr.length > 0) {
      this.platformSend();
      if (this.timer != null) {
        clearTimeout(this.timer);
        this.timer = null;
      }
      this.arr = [];
    }
  }
  sendInner() {
    if (this.timer) {
      if (this.arr.length >= this.count) {
        clearTimeout(this.timer);
        this.timer = null;
        this.sendImmediateInner();
      }
    } else {
      const that = this;
      if (this.arr.length >= this.count || this.time <= 0) {
        this.sendImmediateInner();
      } else {
        this.timer = setTimeout(function() {
          that.sendImmediateInner();
        }, this.time * 1e3);
      }
    }
  }
  send(originLog) {
    const obj = this.transString(originLog);
    this.arr.push(obj);
    this.sendInner();
  }
  sendImmediate(originLog) {
    const obj = this.transString(originLog);
    this.arr.push(obj);
    this.sendImmediateInner();
  }
  sendBatchLogs(originLogs) {
    const logs = originLogs.map((originLog) => this.transString(originLog));
    this.arr.push(...logs);
    this.sendInner();
  }
  sendBatchLogsImmediate(originLogs) {
    const logs = originLogs.map((originLog) => this.transString(originLog));
    this.arr.push(...logs);
    this.sendImmediateInner();
  }
  overwriteTransString(stsPlugin) {
    this.transString = stsPlugin.transString;
  }
  getOpt() {
    return this.opt;
  }
}

var __async = (__this, __arguments, generator) => {
  return new Promise((resolve, reject) => {
    var fulfilled = (value) => {
      try {
        step(generator.next(value));
      } catch (e) {
        reject(e);
      }
    };
    var rejected = (value) => {
      try {
        step(generator.throw(value));
      } catch (e) {
        reject(e);
      }
    };
    var step = (x) => x.done ? resolve(x.value) : Promise.resolve(x.value).then(fulfilled, rejected);
    step((generator = generator.apply(__this, __arguments)).next());
  });
};
function send(url, payload, sdk, platformRequestName) {
  const { request } = getPlatformRequestMethod(sdk, platformRequestName);
  request({
    url: `${url}?APIVersion=0.6.0`,
    method: "POST",
    data: payload
  });
}
function sendPutlog(url, payLoad, sdk, appName, stsPlugin, platformRequestName) {
  return __async(this, null, function* () {
    url = url.slice(0, -6);
    const { data, header } = yield stsPlugin.process(url, payLoad);
    const { request } = getPlatformRequestMethod(sdk, platformRequestName);
    appName && (appName == "alipay" || appName == "dingtalk") ? request({
      url,
      method: "POST",
      data,
      headers: header
    }) : request({
      url,
      method: "POST",
      data,
      header
    });
  });
}
class WebTrackerMini extends WebTracker {
  constructor(opt) {
    const { sdk } = getPlatformSDKInterop(opt);
    const superOpt = Object.assign({}, opt, {
      sendPayload: (url, payload) => {
        send(url, payload, sdk, opt.platformRequestName);
      }
    });
    super(superOpt);
  }
  useStsPlugin(stsPlugin) {
    const { sdk, appName } = getPlatformSDKInterop(this.getOpt());
    this.getOpt().sendPayload = (url, payload) => {
      let optmini = this.getOpt();
      sendPutlog(url, payload, sdk, appName, stsPlugin, optmini.platformRequestName);
    };
    this.overwriteTransString(stsPlugin);
  }
}

export { WebTrackerMini as default, getPlatformRequestMethod, getPlatformSDKInterop };
//# sourceMappingURL=web-track-mini.es.js.map
